import os
from mistune.markdown import preprocess
from .base import Directive


class DirectiveInclude(Directive):
    def parse(self, block, m, state):
        source_file = state.get('__file__')
        if not source_file:
            return {
                'type': 'block_error',
                'raw': 'Missing source file configuration',
            }

        relpath = m.group('value')
        options = self.parse_options(m)

        dest = os.path.join(os.path.dirname(source_file), relpath)
        dest = os.path.normpath(dest)
        if dest == source_file:
            return {
                'type': 'block_error',
                'raw': 'Could not include self: ' + relpath,
            }

        if not os.path.isfile(dest):
            return {
                'type': 'block_error',
                'raw': 'Could not find file: ' + relpath,
            }

        with open(dest, 'rb') as f:
            content = f.read()
            text = content.decode('utf-8')

        if not options:
            ext = os.path.splitext(relpath)[1]
            if ext in {'.md', '.markdown', '.mkd'}:
                text, state = preprocess(text, {'__file__': dest})
                return block.parse(text, state)
            if ext in {'.html', '.xhtml', '.htm'}:
                return {'type': 'block_html', 'text': text}

        return {
            'type': 'include',
            'raw': text,
            'params': (relpath, dest, options)
        }

    def __call__(self, md):
        self.register_directive(md, 'include')
        if md.renderer.NAME == 'html':
            md.renderer.register('include', render_html_include)

        elif md.renderer.NAME == 'ast':
            md.renderer.register('include', render_ast_include)


def render_ast_include(text, relpath, abspath=None, options=None):
    return {
        'type': 'include',
        'text': text,
        'relpath': relpath,
        'abspath': abspath,
        'options': options,
    }


def render_html_include(text, relpath, abspath=None, options=None):
    html = '<section class="directive-include" data-relpath="'
    return html + relpath + '">\n' + text + '</section>\n'



from tests import fixtures
from unittest import TestCase


class BaseTestCase(TestCase):
    @classmethod
    def load_fixtures(cls, case_file):
        def attach_case(n, text, html):
            def method(self):
                self.assert_case(n, text, html)

            name = 'test_{}'.format(n)
            method.__name__ = name
            method.__doc__ = 'Run fixture {} - {}'.format(case_file, n)
            setattr(cls, name, method)

        for n, text, html in fixtures.load_examples(case_file):
            if cls.ignore_case(n):
                continue
            attach_case(n, text, html)

    @classmethod
    def ignore_case(cls, name):
        return False

    def assert_case(self, name, text, html):
        result = self.md(text)
        self.assertEqual(result, html)



from mistune import create_markdown
from mistune.directives import DirectiveToc
from mistune.directives.toc import render_toc_ul, extract_toc_items
from tests import BaseTestCase, fixtures


class TestPluginToc(BaseTestCase):
    @staticmethod
    def parse(text):
        md = create_markdown(
            escape=False,
            plugins=[DirectiveToc()]
        )
        html = md(text)
        return html

    def assert_case(self, name, text, html):
        result = self.parse(text)
        self.assertEqual(result, html)


class TestPluginTocAst(BaseTestCase):
    def test_render_toc_ul(self):
        result = render_toc_ul(None)
        self.assertEqual(result, '')

    def test_extract_toc_items(self):
        md = create_markdown(
            renderer='ast',
            plugins=[DirectiveToc()]
        )
        self.assertEqual(extract_toc_items(md, ''), [])
        s = '# H1\n## H2\n# H1'
        result = extract_toc_items(md, s)
        expected = [
            ('toc_1', 'H1', 1),
            ('toc_2', 'H2', 2),
            ('toc_3', 'H1', 1),
        ]
        self.assertEqual(result, expected)

    def test_ast_renderer(self):
        md = create_markdown(
            renderer='ast',
            plugins=[DirectiveToc()]
        )
        data = fixtures.load_json('toc.json')
        self.assertEqual(md(data['text']), data['tokens'])


TestPluginToc.load_fixtures('toc.txt')



import re
import mistune
from mistune.util import html
from tests import BaseTestCase


IGNORE_CASES = {
    'setext_headings_002',  # we only allow one line title
    'setext_headings_015',

    'setext_headings_003',  # must start with 2 = or -
    'setext_headings_007',  # ignore
    'setext_headings_013',  # ignore

    'html_blocks_039',  # ignore
    'link_reference_definitions_019',  # weird rule

    'block_quotes_008',  # we treat it different

    'list_items_005',  # I don't agree
    'list_items_024',
    'list_items_028',
    'list_items_039',  # no lazy
    'list_items_040',
    'list_items_041',

    'lists_007',  # we use simple way to detect tight list
    'lists_016',
    'lists_017',
    'lists_018',
    'lists_019',

    'block_quotes_005',  # we don't allow lazy continuation
    'block_quotes_006',
    'block_quotes_011',
    'block_quotes_020',
    'block_quotes_023',
    'block_quotes_024',  # this test case shows why lazy is not good

    'code_spans_009',  # code has no priority
    'code_spans_010',

    'entity_and_numeric_character_references_004',  # &entity is allowed
    'entity_and_numeric_character_references_005',

    'links_029',  # emphasis problem
    'links_031',  # different behavior
    'links_034',
    'links_037',
    'links_038',  # code has no priority
    'links_039',
    'links_043',  # emphasis problem
    'links_045',  # different behavior
    'links_046',
    'links_047',
    'links_049',
    'links_050',  # code has no priority
    'links_051',  # different behavior
    'links_064',  # allow empty key
    'links_065',
    'links_077',

    'images_002',  # we just keep everything as raw
    'images_003',
    'images_004',
    'images_005',
    'images_006',
    'images_014',
    'images_018',

    'autolinks_002',  # don't understand
}
INSANE_CASES = {
    'fenced_code_blocks_013',
    'fenced_code_blocks_015',
    'list_items_033',
    'list_items_038',

    'link_reference_definitions_002',  # only allow one line definition
    'link_reference_definitions_003',
    'link_reference_definitions_004',
    'link_reference_definitions_005',
    'link_reference_definitions_007',
    'link_reference_definitions_021',

    'links_025',
    'links_032',
    'links_033',
    'links_041',
    'links_060',
    'links_082',
    'links_084',
}

if html is None:
    PY2_IGNORES = {
        'entity_and_numeric_character_references_001',
        'entity_and_numeric_character_references_002',
        'entity_and_numeric_character_references_003',
        'entity_and_numeric_character_references_008',
        'entity_and_numeric_character_references_009',
        'entity_and_numeric_character_references_010',
        'links_016', 'links_019',
    }
else:
    PY2_IGNORES = []

DIFFERENCES = {
    'tabs_005': lambda s: s.replace('<code>  ', '<code>'),
    'tabs_006': lambda s: s.replace('<code>  ', '<code>'),
    'tabs_007': lambda s: s.replace('<code>  ', '<code>'),
}


PASSED = {
    'tabs', 'thematic', 'atx',
    'setext', 'indented', 'fenced',
    'html_blocks', 'link_ref',
    'paragraphs', 'blank_lines',
    'block_quotes', 'list_items', 'lists',
    'backslash', 'entity', 'code_spans',
    # emphasis
    'links', 'images', 'autolinks', 'raw_html',
    'hard_line', 'soft_line', 'textual',
}


class TestCommonMark(BaseTestCase):
    @classmethod
    def ignore_case(cls, n):
        if n.startswith('emphasis'):
            return True
        if PY2_IGNORES and n in PY2_IGNORES:
            return True
        return (n in IGNORE_CASES) or (n in INSANE_CASES)

    def assert_case(self, n, text, html):
        result = mistune.html(text)
        # normalize to match commonmark
        result = re.sub(r'\s*\n+\s*', '\n', result)
        result = re.sub(r'>\n', '>', result)
        result = re.sub(r'\n<', '<', result)
        expect = re.sub(r'\s*\n+\s*', '\n', html)
        expect = re.sub(r'>\n', '>', expect)
        expect = re.sub(r'\n<', '<', expect)
        if n in DIFFERENCES:
            expect = DIFFERENCES[n](expect)
        self.assertEqual(result, expect)


TestCommonMark.load_fixtures('commonmark.txt')



import os
from mistune import create_markdown
from mistune.directives import DirectiveInclude
from tests.fixtures import ROOT
from unittest import TestCase


class TestPluginDirective(TestCase):
    def test_html_include(self):
        md = create_markdown(
            escape=False,
            plugins=[DirectiveInclude()]
        )
        html = md.read(os.path.join(ROOT, 'include/text.md'))
        self.assertIn('Could not include self', html)
        self.assertIn('Could not find file', html)
        self.assertIn('<div>include html</div>', html)
        self.assertIn('<blockquote>', html)
        self.assertIn('# Table of Contents', html)

    def test_include_missing_source(self):
        md = create_markdown(plugins=[DirectiveInclude()])
        s = '.. include:: foo.txt'
        html = md(s)
        self.assertIn('Missing source file', html)

    def test_ast_include(self):
        md = create_markdown(
            renderer='ast',
            plugins=[DirectiveInclude()]
        )
        filepath = os.path.join(ROOT, 'include/foo.txt')
        s = '.. include:: hello.txt'
        tokens = md.parse(s, {'__file__': filepath})
        token = tokens[0]
        self.assertEqual(token['type'], 'include')
        self.assertEqual(token['text'], 'hello\n')
        self.assertEqual(token['relpath'], 'hello.txt')



import json
from mistune import create_markdown
from mistune.directives import Admonition
from tests import fixtures
from unittest import TestCase


class TestPluginAdmonition(TestCase):
    def test_unsupported_directive(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. hello:: Warnning\n\n   message'
        html = md(s)
        self.assertIn('Unsupported directive: hello', html)

    def test_note_admonition(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. note:: Warnning\n\n   message'
        html = md(s)
        self.assertIn('class="admonition note"', html)
        self.assertIn('<p class="admonition-title">Warnning</p>', html)
        self.assertIn('<p>message</p>', html)

    def test_code_admonition(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. note:: Warnning\n\n       print() '
        html = md(s)
        self.assertIn('class="admonition note"', html)
        self.assertIn('<p class="admonition-title">Warnning</p>', html)
        self.assertIn('<pre><code>print() \n</code></pre>', html)

    def test_note_admonition_no_text(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. note:: Warnning'
        html = md(s)
        self.assertIn('class="admonition note"', html)
        self.assertIn('<p class="admonition-title">Warnning</p>', html)

    def test_admonition_options(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. note:: Warnning\n\n   :option: 3'
        html = md(s)
        self.assertIn('Admonition has no options', html)

    def test_missing_title(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. note::\n\n   message'
        html = md(s)
        self.assertIn('<p class="admonition-title">Note</p>', html)

    def test_empty_title(self):
        md = create_markdown(plugins=[Admonition()])
        s = '.. note:: ""\n\n   message'
        html = md(s)
        self.assertNotIn('<p class="admonition-title">Note</p>', html)

    def test_ast_admonition(self):
        data = fixtures.load_json('admonition.json')
        md = create_markdown(renderer='ast', plugins=[Admonition()])
        # Use JSON to fix the differences between tuple and list
        tokens = json.loads(json.dumps(md(data['text'])))
        self.assertEqual(tokens, data['tokens'])
